import ooeuler_getter

class P2WithParallelogramFundamentalRegion(ooeuler_getter.Polyhedron):
    def __init__(self,opt):
        """
        Note that the union of
        { (i,j) | i=0,..,n_row-1,  j=0,..,n_col-1} and { (n_row,0) }
        is the complete system of representatives.
        """
        (n_row,n_col)=opt
        self.n_row=n_row
        self.n_col=n_col

        self.faces=[ (i,j) for i in range(n_row) for j in range(n_col) ]
        self.faces.append((n_row,0))

        rep={}
        for i in range(n_row+1):
            for j in range(n_col+1):
                rep[(i,j)] = self.get_equivalent_cell((i,j))

        self.edges=[]
        for i in range(n_row):
            for j in range(n_col):
                e=[rep[(i,j)], rep[(i,j+1)]]
                self.edges.append(frozenset(e))
                e=[rep[(i,j)], rep[(i+1,j)]]
                self.edges.append(frozenset(e))
                e=[rep[(i,j)], rep[(i+1,j+1)]]
                self.edges.append(frozenset(e))
        e=[rep[(n_row-1,0)], rep[(0,n_col-1)],None]
        self.edges.append(frozenset(e))

        self.vertices=[]
        for i in range(n_row):
            for j in range(n_col):
                v=[rep[(i,j)], rep[(i,j+1)], rep[(i+1,j+1)]]
                self.vertices.append(frozenset(v))
                v=[rep[(i,j)], rep[(i+1,j)], rep[(i+1,j+1)]]
                self.vertices.append(frozenset(v))
        v=[rep[(n_row-1,0)], rep[(0,n_col-1)], rep[(n_row,0)], None]
        self.vertices.append(frozenset(v))

        self.face_as_edges={} 
        self.face_as_vertices={}
        for f in self.faces:
            es = [ e for e in self.edges if f in e ]
            self.face_as_edges[f]=frozenset(es)
            vs = [ v for v in self.vertices if f in v ]
            self.face_as_vertices[f]=frozenset(vs)
        self.edges=frozenset(self.edges)
        self.vertices=frozenset(self.vertices)

        chart=[[self.get_equivalent_cell((i,j)) for j in range(n_col+1)] for i in range(n_row+1)]
        self.layout_hint=[chart]
        self.layout_hint_type="multilex"

    def get_equivalent_cell(self,cell):
        (i,j)=cell
        if i<self.n_row:
            if j<self.n_col:
                return (i,j)
            else:
                return (self.n_row-i,0)
        else:
            if j==0:
                return (i,0)
            else:
                return (0,self.n_col-j)

